﻿// TextVariablesShow.jsx

/*
    
Программа выясняет, какие в открытом документе текстовые переменные используются, и выводит их список.
После этого, выбрав переменную, можно узнать, на какой странице она есть, и какой в ней текст.
В этой версии предполагается, что фреймы с текстовыми переменными в группы не входят.

Мысль, что такой сервис может быть полезен, появилась в сообщении на rudtp:
https://forum.rudtp.ru/threads/pokazat-ispolzovanie-tekstovyx-peremennyx-v-tekste.80119/#post-1340182

-----

https://dotextok.ru   |   dotextok@gmail.com

*/

#targetengine "TextVariablesShow" 

var programTitul = "Просмотр текстовых переменных"; //  версия15.03.2023

// Найденные тексты текстовых переменных собираются в отдельные записи. Формат записи <номер страницы><разделитель sepIn><текст переменной>
// Записи помещаются одна за другой в строку. Последовательность записей в строке: 
// <номер страницы><разделитель sepIn><текст переменной1><разделитель sepOut><номер страницы><разделитель sepIn><текст переменной2><разделитель sepOut><номер страницы><разделитель sepIn><текст переменной3>
var sepIn = ">>:::<<" ; // разделитель между номером страницы и текстом внутри одной записи.
var sepOut = "#>##<#";  // разделитель между записями в строке. 
// Чем выше абракадабристость этих разделителей, тем меньше вероятность, что такое сочетание знаков неожиданно встретится в реальном тексте
var overriden = "[*] "; //  указатель, что этот фрейм отвязан от мастер-страницы
var texVarListLength = 400; // длина строки в выпадающем списке текстовых переменных
///////////////////////////////
var ProgressBar = function(title) { // ProgressBar
var w = new Window('palette', title, {x:0, y:0, width:740, height:60},{closeButton: true});
pb = w.add('progressbar', {x:20, y:32, width:700, height:12});
st = w.add('statictext', {x:20, y:12, width:700, height:20});
st.justify = 'left';
w.center();
this.reset = function(msg,maxValue) {
st.text = msg;
pb.value = 0;
pb.maxvalue = maxValue;
w.show();
w.update();
}
this.hit = function() {++pb.value; w.update();}
this.hide = function() {w.hide();} // эта функция гасит окно процесса исполнения задания
this.close = function() {w.close();}
this.info = function(msg)  {
    st.text = msg;
    w.show();
    w.update();
    }
} // ProgressBar
//////

if (app.documents.length == 0) {
    alert("Откройте файл перед запуском этой программы.", programTitul);
    exit();
    }
var doc = app.activeDocument;
var textVars = doc.textVariables;

var textVarsList = [];
var cur_tp;
for (i = 0; i < textVars.length; i++) { // i++
    cur_tp = textVars[i];
    if (cur_tp.variableType == VariableTypes.XREF_PAGE_NUMBER_TYPE || cur_tp.variableType == VariableTypes.XREF_CHAPTER_NUMBER_TYPE) continue; // служебные текстовые переменные, всегда присутствующие в файле
   if (cur_tp.associatedInstances.length == 0) continue; // эта переменная не используется
    textVarsList.push(cur_tp.name);
	} // i++
if (textVarsList.length == 0) {
    alert("В этой работе текстовые переменные не используются.", programTitul);
    exit();
    }  
var tpInfo = {};
var pagesLength = doc.pages.length;
var page, masterItems, tpName, theItem, bounds, xB, yB, pageNum, tpText;
var pBar = new ProgressBar(programTitul);
var pMes = "Извлечение информации о текстовых переменных ";
pBar.reset(pMes, pagesLength);
for (p = 0; p < pagesLength; p++, pBar.hit()) { // p++
     page = doc.pages[p];
     pBar.info(pMes + " | " + (p+1) + "/" + pagesLength);
     // сначала проверка неотвязанных от мастер-страницы объектов
     masterItems = page.masterPageItems.length;
     if (masterItems > 0) { // > 0
        for (i = masterItems-1; i >=0 ; i--) { // i--
            theItem = page.masterPageItems[i]; 
            try { 
                if (theItem.texts[0].textVariableInstances.length == 0) continue; // этот объект не содержит текстовую переменную - следующий i-цикл
                }
            catch (e) { continue; } // это графический объект или группа - следующий i-цикл
            bounds = theItem.geometricBounds;
            xB = bounds[1];   
            yB = bounds[0];  // запомнили координаты фрейма, когда он ещё принадлежит мастер-странице ..>..    
            theItem = page.masterPageItems[i].override(page); // ...>   отсоединили от мастер-страницы...
            theItem.move([xB,yB]); // ..>.. эта строка возвращает объект на место, если он при отсоединении сместился.
            tpName = theItem.textVariableInstances[0].name; // имя текстовой переменной        
            tpText = theItem.textVariableInstances[0].resultText; // это текст данной текстовой переменной на этой странице
            pageNum = page.name; // номер страницы
            theItem.removeOverride(); // ... >  вернули фрейм с текстовой переменной на мастер-страницу
            // сохраняем полученную информацию:
            if (tpInfo[tpName] == undefined) tpInfo[tpName] = pageNum + sepIn+ tpText;
            else tpInfo[tpName] = tpInfo[tpName] + sepOut + pageNum + sepIn+ tpText;
            } // i--   
        } // > 0
    // теперь поиск отвязанных от мастер-страницы текстовых фреймов, в которых есть текстовые переменные
    if (page.textFrames.length > 0) { /// > 0
        for (ii = page.textFrames.length-1; ii >= 0; ii--) { // ii--
            theItem = page.textFrames[ii];
            if (theItem.textVariableInstances.length == 0) continue;
            tpName = theItem.textVariableInstances[0].name;
            tpText = theItem.textVariableInstances[0].resultText;
            pageNum = page.name;
            if (tpInfo[tpName] == undefined) tpInfo[tpName] = pageNum + sepIn+ overriden + tpText;
            else tpInfo[tpName] = tpInfo[tpName] + sepOut + pageNum +  sepIn+ overriden + tpText; // overriden = [*] -- указатель, что этот фрейм отвязан от мастер-страницы            
            } // ii--
        } /// > 0
    } // p++
pBar.close();
////////////////////////
var w = new Window ("palette",programTitul);
w.alignChildren = ["fill", "fill"];
w.orientation = "column";
var listInfoRow = w.add("group");
listInfoRow.orientation = "row";
var textVarList = listInfoRow.add ("dropdownlist", undefined, textVarsList);
textVarList.minimumSize.width = textVarList.maximumSize.width = texVarListLength;
textVarList.selection = 0;
var info = listInfoRow.add("button", [0,0,28,28], "?");
separator0 = w.add ("panel");
separator0.minimumSize.height = separator0.maximumSize.height = 1;
var tvInfo = w.add ("edittext", [0,0,texVarListLength,200], "Выберите одну из текстовых переменных.", {multiline: true, scrolling: true}); // "edittext" - чтобы работала опция скроллинга
///////////////////////////////////////////
textVarList.onActivate = function () { // textVarList.onActivate
var tpName = String(textVarList.selection);
var tpData = tpInfo[tpName];
regExp1 = new RegExp(sepIn,"g");
tpData = tpData.replace(regExp1, ": "); 
regExp2 = new RegExp(sepOut,"g");
tpData = tpData.replace(regExp2, "\n"); 
tvInfo.text = "";
tvInfo.text = tpData;
return;
} // textVarList.onActivate
///
info.onClick = function () { // info.onClick
var theInfo = "Программа выясняет, какие в открытом документе текстовые переменные используются, и выводит их список.\nПосле этого, выбрав переменную, можно узнать, на каких страницах она присутствует, и какой там в ней текст.\nЕсли фрейм с текстовой переменной отвязан от мастер-страницы, то перед его текстом будет звёздочка в квадратных скобках ( [*] ).\n\nВ этой версии предполагается, что фреймы с текстовыми переменными в группы не входят.\n\nМысль, что такой сервис может быть полезен, появилась тут:\nhttps://forum.rudtp.ru/threads/pokazat-ispolzovanie-tekstovyx-peremennyx-v-tekste.80119/#post-1340182\n\n-----\n\nhttps://dotextok.ru   |   dotextok@gmail.com\n";
alert(theInfo, programTitul);
} // info.onClick
////
w.onClose = function() { // w.onClose
tpInfo = null;
}  // w.onClose
////
w.show();


    
